package lightcontainer.protocol.rules.reader;

import lightcontainer.protocol.ProtocolReader;
import lightcontainer.utils.FileReceiver;

import java.io.IOException;
import java.io.InputStream;

public class SavefileRule extends ProtocolReader {
    // Constants
    private static final String PATTERN = "^SAVEFILE ([^ !]{1,20}) ([0-9]{1,10})\r\n$";
    private static final int FILE_NAME = 0; // Index file name.
    private static final int FILE_SIZE = 1; // Index file size.

    // Constructor
    public SavefileRule() {
        super(PATTERN);
    }

    public class Result extends ProtocolResult {
        // Variables
        private final String filename;
        private final int size;

        // Construct
        public Result(String filename, int size) {
            this.filename = filename;
            this.size = size;
        }

        @Override
        public void read(InputStream reader) {
            super.read(reader);
            System.out.printf("Sauvegarde du fichier : %s %d\n", filename, size);

            try {
                FileReceiver fileReceiver = new FileReceiver("D:\\");
                if (!fileReceiver.receiveFile(reader, this.filename, this.size))
                    throw new IOException();

                this.setResultCommand("SAVEFILE_OK\r\n", ResultCmdReceiver.CLIENT);
            } catch (IOException e) {
                this.setResultCommand("SAVEFILE_ERROR\r\n", ResultCmdReceiver.CLIENT);
                e.printStackTrace();
            }
        }
    }

    /**
     * Cette méthode est appelée lors de l'exécution de la règle
     * @param data Paramètres pour créer la commande.
     */
    @Override
    protected SavefileRule.Result onExecuted(String... data) {
        SavefileRule.Result result = new SavefileRule.Result(data[FILE_NAME], Integer.parseInt(data[FILE_SIZE]));

        return result;
    }
}
